/*
 * IRQ test program (on console)
 */
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <stdio.h>

#include "rex5055d.h"

void  CardInit(int argc, char **argv);
void  CardRelease(void);
int   DataIn( int * );

static char *Card_dev = "/dev/dio";
static int      Card_fd;

/* ================================================== */

main(int argc, char **argv)
{
    int  count;
    int  val;

    CardInit(argc, argv);
    for ( count = 0; count < 50; count++ ) {
        printf("Count:%3d ", count);
        if (DataIn(&val) & 0x80)
            printf("Data:%04X\n", val );
        printf("\n");
        sleep(1);
    }
    CardRelease();
}

/*
 * Open /dev/dio* &
 * Initialize REX-5055 Ctrl-reg, IRQMask-reg
 */
void
CardInit(int argc, char **argv)
{
    int   Ctrl_val = 0x88;    /* mode = low & high input mode */
                                           /* up edge , PIO-1 */
    int   Mask_val = 0x82;    /* Int enable, high-in for clear */

    if (argc > 1)
        Card_dev = argv[1];
    else
        Card_dev = "/dev/dio";

    Card_fd = open(Card_dev, O_RDWR);
    if (Card_fd == -1) {
        printf("%s open error ...\n", Card_dev);
        exit(1);
    }

     /* Set Contorl */
    if ( ioctl(Card_fd, REXDIO_SETCTRL, &Ctrl_val) == 1) {
        printf("%s REXDIO_SETCTRL error...\n", Card_dev);
        exit(1);
    }

     /* Set Int mask */
    if ( ioctl(Card_fd, REXDIO_SETMASK, &Mask_val) == -1) {
        printf("%s REXDIO_SETMASK error...\n", Card_dev);
        exit(1);
    }

}

/*
 * Close device file 
 */
void
CardRelease(void)
{
    close(Card_fd);
}

/*
 * Check IRQ & Get Data
 */
int
DataIn( int *val )
{
    struct arg {
        short int   Stat_val;
        short int   Word_val;
    };
    struct arg  dio_arg = { 0, 0};

    if ( ioctl(Card_fd, REXDIO_IRQSTAT, &dio_arg) == -1) {
        printf("%s REXDIO_IRQSTAT error...\n", Card_dev);
        exit(1);
    }
    if (dio_arg.Stat_val & 0x80)
        *val = dio_arg.Word_val;

    return dio_arg.Stat_val;
}
